(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core'), require('@angular/common'), require('@angular/forms'), require('@angular/fire/storage'), require('@angular/material'), require('@angular/flex-layout'), require('rxjs/operators')) :
	typeof define === 'function' && define.amd ? define(['exports', '@angular/core', '@angular/common', '@angular/forms', '@angular/fire/storage', '@angular/material', '@angular/flex-layout', 'rxjs/operators'], factory) :
	(factory((global['file-store-lib'] = {}),global.core,global.common,global.forms,global.storage,global.material,global.flexLayout,global.operators));
}(this, (function (exports,core,common,forms,storage,material,flexLayout,operators) { 'use strict';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */
var FileUploadComponent = /** @class */ (function () {
    function FileUploadComponent(storage$$1) {
        this.storage = storage$$1;
        /**
         * By default developmentMode is false
         * Set this to true to observe log outputs on console
         */
        this.developmentMode = false;
        /**
         * Output event onUpload($event:FileStoreRecord) emitted upon completion of upload task
         */
        this.onUpload = new core.EventEmitter();
        this.showProgress = false;
        this.uploadMetadata = { cacheControl: '' };
    }
    /**
     * @return {?}
     */
    FileUploadComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        // Throw Error, if path in config is missing
        if (!this.config || !this.config.path) {
            throw new Error('[file-store-lib] Error: Missing path in File Store Configuration.');
        }
        /** @type {?} */
        var maxAge = this.config.cacheMaxAge ? this.config.cacheMaxAge : 60 * 60 * 24 * 365;
        this.uploadMetadata.cacheControl = 'maxage=' + maxAge;
        this.config.placeholder = this.config.placeholder ? this.config.placeholder : 'Select a file';
        this.config.accept = this.config.accept ? this.config.accept : '*.*';
        this.uploadButtonName = this.config.uploadButtonName ? this.config.uploadButtonName.toLocaleUpperCase() : 'UPLOAD';
    };
    // trigger on completion of file selection by the user in the view
    /**
     * @param {?} e
     * @return {?}
     */
    FileUploadComponent.prototype.inputChanged = /**
     * @param {?} e
     * @return {?}
     */
    function (e) {
        this.file = e.dataTransfer ? e.dataTransfer.files[0] : e.target.files[0];
        this.fileName = this.file ? this.file.name : '';
        this.errorFeedback = ''; // Reset any previous errors
        this.successFeedback = ''; // Reset any previous completions
    };
    // trigger on click of upload user action in the view
    /**
     * @return {?}
     */
    FileUploadComponent.prototype.upload = /**
     * @return {?}
     */
    function () {
        this.consoleLog('Upload file attempted!');
        this.successFeedback = ''; // Reset any previous completions
        if (this.file) {
            this.consoleLog('File to upload: ', this.file);
            this.uploadFile(this.file);
        }
        else {
            this.errorFeedback = 'No file selected yet to upload!';
            this.consoleLog(this.errorFeedback);
        }
    };
    /**
     * @return {?}
     */
    FileUploadComponent.prototype.uploadComplete = /**
     * @return {?}
     */
    function () {
        this.successFeedback = this.fileName + ': upload complete.';
        this.file = null; // Reset the uploaded file reference to avoid duplicate upload
        this.fileName = ''; // Resetting the file name on the View
        this.showProgress = false;
        this.onUpload.emit(this.fileStoreRecord);
    };
    /**
     * @param {?} file
     * @return {?}
     */
    FileUploadComponent.prototype.uploadFile = /**
     * @param {?} file
     * @return {?}
     */
    function (file) {
        var _this = this;
        this.consoleLog('Uploading File: ', file.name);
        // initialize the view state
        this.showProgress = true;
        this.errorFeedback = '';
        /** @type {?} */
        var storageUploadPath = this.config.path + "/" + file.name;
        /** @type {?} */
        var fileRef = this.storage.ref(storageUploadPath);
        /** @type {?} */
        var task = this.storage.upload(storageUploadPath, file, this.uploadMetadata);
        this.uploadPercent = task.percentageChanges();
        task
            .snapshotChanges()
            .pipe(operators.finalize(function () {
            fileRef.getDownloadURL().subscribe(function (baseUrl) {
                /** @type {?} */
                var url = new URL(baseUrl);
                /** @type {?} */
                var fileStoreRecord = {
                    type: file.type,
                    name: file.name,
                    downloadURL: url,
                    createdOn: new Date(),
                    md5Hash: 'None' // metadata.md5Hash // TODO: Refactor to get the metadata and Hash
                };
                _this.fileStoreRecord = fileStoreRecord;
                _this.uploadComplete();
            });
        }))
            .subscribe();
    };
    /**
     * @param {?=} msg
     * @param {...?} optionalParams
     * @return {?}
     */
    FileUploadComponent.prototype.consoleLog = /**
     * @param {?=} msg
     * @param {...?} optionalParams
     * @return {?}
     */
    function (msg) {
        var optionalParams = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            optionalParams[_i - 1] = arguments[_i];
        }
        if (this.developmentMode) {
            if (optionalParams) {
                console.log('[file-store-lib] ', msg, optionalParams);
            }
            else {
                console.log('[file-store-lib] ', msg);
            }
        }
    };
    FileUploadComponent.decorators = [
        { type: core.Component, args: [{
                    selector: 'mlc-file-upload',
                    template: "<div fxLayout=\"column\" fxLayoutGap=\"12px\" fxFlex> <div fxLayout=\"row wrap\" fxLayoutGap=\"8px\" fxLayoutAlign=\"start center\"> <input #inputFile type=\"file\" (change)=\"inputChanged($event)\" [accept]=\"config.accept\" fxHide> <mat-form-field> <input matInput [(ngModel)]=\"fileName\" (click)=\"inputFile.click()\" [placeholder]=\"config.placeholder\" disabled> </mat-form-field> <button mat-mini-fab (click)=\"inputFile.click()\"><mat-icon class=\"md-24\">attach_file</mat-icon></button> </div> <div> <mat-hint>{{successFeedback}}</mat-hint> <mat-error>{{errorFeedback}}</mat-error> </div> <mat-progress-bar *ngIf=\"showProgress\" [value]=\"uploadPercent | async\"> </mat-progress-bar> <div fxLayout=\"row\" fxLayoutAlign=\"end center\"> <ng-content></ng-content> <div fxFlex></div> <button mat-raised-button (click)=\"upload()\" color=\"primary\" [disabled]=\"showProgress\">{{uploadButtonName}}</button> </div> </div>"
                },] },
    ];
    /** @nocollapse */
    FileUploadComponent.ctorParameters = function () { return [
        { type: storage.AngularFireStorage }
    ]; };
    FileUploadComponent.propDecorators = {
        config: [{ type: core.Input }],
        developmentMode: [{ type: core.Input }],
        onUpload: [{ type: core.Output }]
    };
    return FileUploadComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */
var FileDropDirective = /** @class */ (function () {
    function FileDropDirective() {
        this.filesDropped = new core.EventEmitter();
        this.filesHovered = new core.EventEmitter();
    }
    /**
     * @param {?} $event
     * @return {?}
     */
    FileDropDirective.prototype.onDrop = /**
     * @param {?} $event
     * @return {?}
     */
    function ($event) {
        $event.preventDefault();
        /** @type {?} */
        var transfer = $event.dataTransfer;
        this.filesDropped.emit(transfer.files);
        this.filesHovered.emit(false);
    };
    /**
     * @param {?} $event
     * @return {?}
     */
    FileDropDirective.prototype.onDragOver = /**
     * @param {?} $event
     * @return {?}
     */
    function ($event) {
        event.preventDefault();
        this.filesHovered.emit(true);
    };
    /**
     * @param {?} $event
     * @return {?}
     */
    FileDropDirective.prototype.onDragLeave = /**
     * @param {?} $event
     * @return {?}
     */
    function ($event) {
        this.filesHovered.emit(false);
    };
    FileDropDirective.decorators = [
        { type: core.Directive, args: [{
                    selector: '[fileDrop]'
                },] },
    ];
    /** @nocollapse */
    FileDropDirective.ctorParameters = function () { return []; };
    FileDropDirective.propDecorators = {
        filesDropped: [{ type: core.Output }],
        filesHovered: [{ type: core.Output }],
        onDrop: [{ type: core.HostListener, args: ['drop', ['$event'],] }],
        onDragOver: [{ type: core.HostListener, args: ['dragover', ['$event'],] }],
        onDragLeave: [{ type: core.HostListener, args: ['dragleave', ['$event'],] }]
    };
    return FileDropDirective;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */
/**
 * File Drag & Drop Component
 */
var FileDndComponent = /** @class */ (function () {
    function FileDndComponent(storage$$1) {
        this.storage = storage$$1;
        /**
         * By default developmentMode is false
         * Set this to true to observe log outputs on console
         */
        this.developmentMode = false;
        /**
         * Output event onUpload($event:FileStoreRecord[]) emitted upon completion of upload task(s)
         */
        this.onUpload = new core.EventEmitter();
        this.uploadButtonPushCount = 0;
        this.uploadMetadata = { cacheControl: '' };
    }
    /**
     * Initialize with defaults and throw error for any missing require configuration
     */
    /**
     * Initialize with defaults and throw error for any missing require configuration
     * @return {?}
     */
    FileDndComponent.prototype.ngOnInit = /**
     * Initialize with defaults and throw error for any missing require configuration
     * @return {?}
     */
    function () {
        this.dropzoneActive = false;
        this.targetFileList = [];
        this.fileStoreRecords = [];
        this.successFeedback = '';
        this.errorFeedback = '';
        // Throw Error, if path in config is missing
        if (!this.config || !this.config.path) {
            this.consoleLog('Error: Missing path in FileStoreConfig');
            throw new Error('[file-store-lib] Error: Missing path in File Store Configuration.');
        }
        /** @type {?} */
        var maxAge = this.config.cacheMaxAge ? this.config.cacheMaxAge : 60 * 60 * 24 * 365;
        this.uploadMetadata.cacheControl = 'maxage=' + maxAge;
        this.uploadButtonName = this.config.uploadButtonName ? this.config.uploadButtonName.toLocaleUpperCase() : 'UPLOAD';
    };
    /**
     * Update dropzoneActive when Dragged & Hovered
     * @param  boolean value based on the Drag Hover
     */
    /**
     * Update dropzoneActive when Dragged & Hovered
     * @param {?} $event
     * @return {?}
     */
    FileDndComponent.prototype.dropzoneState = /**
     * Update dropzoneActive when Dragged & Hovered
     * @param {?} $event
     * @return {?}
     */
    function ($event) {
        this.dropzoneActive = $event;
    };
    /**
     * Upon drop event, upload the files in the FileList
     * @param fileList List of Files to upload
     */
    /**
     * Upon drop event, upload the files in the FileList
     * @param {?} fileList List of Files to upload
     * @return {?}
     */
    FileDndComponent.prototype.handleDrop = /**
     * Upon drop event, upload the files in the FileList
     * @param {?} fileList List of Files to upload
     * @return {?}
     */
    function (fileList) {
        if (!fileList || !fileList.length) {
            return;
        }
        this.consoleLog('FileList after the drop = ', fileList);
        // reset the target array from any previous state
        this.targetFileList = [];
        // First List the files and prepare the progress-bars
        for (var i = 0; i < fileList.length; i++) {
            this.targetFileList.push({
                file: fileList.item(i),
                progress: null,
                downloadURL: null
            });
        }
        this.consoleLog('Target FileList after the drop = ', this.targetFileList);
    };
    /**
     * Upload the dropped target files in the list
     */
    /**
     * Upload the dropped target files in the list
     * @return {?}
     */
    FileDndComponent.prototype.upload = /**
     * Upload the dropped target files in the list
     * @return {?}
     */
    function () {
        var _this = this;
        if (!this.targetFileList.length) {
            this.uploadButtonPushCount = 0;
            return;
        }
        this.uploadButtonPushCount++;
        // Do not repeat the upload, if the button is pressed more than once
        if (this.uploadButtonPushCount > 1) {
            return;
        }
        this.successFeedback = '';
        this.errorFeedback = '';
        this.targetFileList.forEach(function (item) {
            /** @type {?} */
            var storageUploadPath = _this.config.path + "/" + item.file.name;
            /** @type {?} */
            var fileRef = _this.storage.ref(storageUploadPath);
            /** @type {?} */
            var task = _this.storage.upload(storageUploadPath, item.file, _this.uploadMetadata);
            item.progress = task.percentageChanges();
            // TODO: Use flatMap or switchMap and then subscribe, instead of two subscriptions
            task
                .snapshotChanges()
                .pipe(operators.finalize(function () {
                fileRef.getDownloadURL().subscribe(function (baseUrl) {
                    /** @type {?} */
                    var url = new URL(baseUrl);
                    /** @type {?} */
                    var fileStoreRecord = {
                        type: item.file.type,
                        name: item.file.name,
                        downloadURL: url,
                        createdOn: new Date(),
                        md5Hash: 'None' // metadata.md5Hash // TODO: Refactor to get the metadata & md5Hash
                    };
                    _this.fileStoreRecords.push(fileStoreRecord);
                    if (_this.fileStoreRecords.length === _this.targetFileList.length) {
                        _this.uploadComplete();
                    }
                });
            }))
                .subscribe();
        });
    };
    /**
     * Clear the current list of files for upload
     */
    /**
     * Clear the current list of files for upload
     * @return {?}
     */
    FileDndComponent.prototype.clear = /**
     * Clear the current list of files for upload
     * @return {?}
     */
    function () {
        this.targetFileList = [];
        this.fileStoreRecords = [];
        this.uploadButtonPushCount = 0;
        this.successFeedback = '';
        this.errorFeedback = '';
    };
    /**
     * Set success feedback on the view upon completion of all uploads
     * @return {?}
     */
    FileDndComponent.prototype.uploadComplete = /**
     * Set success feedback on the view upon completion of all uploads
     * @return {?}
     */
    function () {
        /** @type {?} */
        var message = 'Upload complete: ' + this.fileStoreRecords.length + ' files uploaded successfully.';
        this.consoleLog(message);
        this.successFeedback = message;
        this.onUpload.emit(this.fileStoreRecords);
    };
    /**
     * Log the outputs to console when developmentMode is true
     * @param {?=} msg Message text to log
     * @param {...?} optionalParams any additional parameters / objects to log along with the message
     * @return {?}
     */
    FileDndComponent.prototype.consoleLog = /**
     * Log the outputs to console when developmentMode is true
     * @param {?=} msg Message text to log
     * @param {...?} optionalParams any additional parameters / objects to log along with the message
     * @return {?}
     */
    function (msg) {
        var optionalParams = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            optionalParams[_i - 1] = arguments[_i];
        }
        if (this.developmentMode) {
            if (optionalParams) {
                console.log('[file-store-lib] DND: ', msg, optionalParams);
            }
            else {
                console.log('[file-store-lib] DND: ', msg);
            }
        }
    };
    FileDndComponent.decorators = [
        { type: core.Component, args: [{
                    selector: 'mlc-file-dnd',
                    template: "<div fxLayout=\"column\" fxFlex fxLayoutGap=\"12px\"> <div fxLayout=\"column\" fxLayoutAlign=\"center center\" fxFlex class=\"dropzone\" fileDrop (filesDropped)=\"handleDrop($event)\" (filesHovered)=\"dropzoneState($event)\" [ngClass]=\"{'active': dropzoneActive}\" *ngIf=\"!targetFileList.length; else showSelectedFiles\"> <div fxFlex> <mat-icon class=\"mat-48\" color=\"primary\">cloud_upload</mat-icon> </div> <div class=\"dropzone-instruction\">Drag and drop the files here</div> </div> <div fxLayout=\"row\" fxLayoutAlign=\"end center\" fxLayoutGap=\"12px\"> <ng-content></ng-content> <div fxFlex></div> <button mat-button (click)=\"clear()\" [disabled]=\"!targetFileList.length\">CLEAR</button> <button mat-raised-button (click)=\"upload()\" color=\"primary\" [disabled]=\"!targetFileList.length\">{{uploadButtonName}}</button> </div> </div> <ng-template #showSelectedFiles> <div fxLayout=\"column\" *ngIf=\"targetFileList.length\" fxLayoutGap=\"12px\"> <mat-grid-list cols=\"5\" rowHeight=\"4:1\"> <mat-grid-tile class=\"table-header\" colspan=\"2\">File</mat-grid-tile> <mat-grid-tile class=\"table-header\">Type</mat-grid-tile> <mat-grid-tile class=\"table-header\">Date</mat-grid-tile> <mat-grid-tile class=\"table-header\">Status</mat-grid-tile> <div *ngFor=\"let item of targetFileList\"> <mat-grid-tile class=\"table-cell\" colspan=\"2\"> <strong>{{item.file.name}}</strong> </mat-grid-tile> <mat-grid-tile class=\"table-cell\"> <i>{{item.file.type}}</i> </mat-grid-tile> <mat-grid-tile class=\"table-cell\">{{item.file.lastModifiedDate | date:'short'}}</mat-grid-tile> <mat-grid-tile class=\"table-cell\"> <mat-icon *ngIf=\"!item.progress\" color=\"primary\">cloud_queue</mat-icon> <div *ngIf=\"item.progress | async; let progress;\"> <mat-progress-spinner diameter=\"32\" [value]=\"progress\" *ngIf=\"progress<100\"></mat-progress-spinner> <mat-icon *ngIf=\"progress===100\" color=\"primary\">check_circle</mat-icon> </div> </mat-grid-tile> </div> </mat-grid-list> <div fxLayout=\"column\"> <p *ngIf=\"!successFeedback\">Total of <strong>{{targetFileList.length}} file(s)</strong> selected for upload.</p> <mat-hint>{{successFeedback}}</mat-hint> <mat-error>{{errorFeedback}}</mat-error> </div> </div> </ng-template>",
                    styles: [".dropzone { border: 4px darkgray dashed; min-height: 100px; } .active { border: 4px darkgray solid; background-color: lightgray; } .table-header { font-size: 80%; font-weight: 600; background-color: lightgray; } .table-cell { font-size: 70%; background-color: lightcyan; } .mat-48 { font-size: 48px; height: auto; width: auto; margin: 12px 0px 0px 0px; } .dropzone-instruction { margin-bottom: 12px; font-weight: 300; } "]
                },] },
    ];
    /** @nocollapse */
    FileDndComponent.ctorParameters = function () { return [
        { type: storage.AngularFireStorage }
    ]; };
    FileDndComponent.propDecorators = {
        config: [{ type: core.Input }],
        developmentMode: [{ type: core.Input }],
        onUpload: [{ type: core.Output }]
    };
    return FileDndComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */
var FileUploadModule = /** @class */ (function () {
    function FileUploadModule() {
    }
    FileUploadModule.decorators = [
        { type: core.NgModule, args: [{
                    imports: [
                        common.CommonModule,
                        forms.FormsModule,
                        forms.ReactiveFormsModule,
                        storage.AngularFireStorageModule,
                        material.MatInputModule, material.MatButtonModule, material.MatIconModule, material.MatProgressSpinnerModule, material.MatProgressBarModule, material.MatGridListModule,
                        flexLayout.FlexLayoutModule
                    ],
                    exports: [FileUploadComponent, FileDndComponent, FileDropDirective],
                    declarations: [FileUploadComponent, FileDndComponent, FileDropDirective]
                },] },
    ];
    return FileUploadModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */
/**
 * FileStoreRecord class represents an equivalent record
 * for every file uploaded to the Firebase Storage
 */
var FileStoreRecord = /** @class */ (function () {
    function FileStoreRecord() {
    }
    return FileStoreRecord;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */

exports.FileUploadModule = FileUploadModule;
exports.FileUploadComponent = FileUploadComponent;
exports.FileDropDirective = FileDropDirective;
exports.FileDndComponent = FileDndComponent;
exports.FileStoreRecord = FileStoreRecord;

Object.defineProperty(exports, '__esModule', { value: true });

})));
